/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - March 20, 2001.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2001
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -  Henk-Johan van Rantwijk <bugs4hj@netscape.net> (Original Author)
   -  Rajnish Bhaskar <raj@lordofthemoon.com>
   -
   - Alternatively, the contents of this file may be used under the
   - terms of the GNU General Public License Version 2 or later (the
   - "GPL"), in which case the provisions of the GPL are applicable
   - instead of those above.  If you wish to allow use of your
   - version of this file only under the terms of the GPL and not to
   - allow others to use your version of this file under the MPL,
   - indicate your decision by deleting the provisions above and
   - replace them with the notice and other provisions required by
   - the GPL.  If you do not delete the provisions above, a recipient
   - may use your version of this file under either the MPL or the
   - GPL.
   -
*/

////////////////////////////////////////////////////////////////////////////////
// this function reads the preference setting for the given menu identifier
///////////////////////////////////////////////////////////////////////////////
function initGroupPref( menuElement )
{
  var prefGroup    = menuElement.getAttribute( "prefgroup" );
  var prefElements = document.getElementsByAttribute( "prefgroup", prefGroup );

  if ( prefElements.length == 0 )
    return;

  var prefType        = prefElements[ 1 ].getAttribute( "preftype" );
  var prefDefval      = prefElements[ 1 ].getAttribute( "prefdefval" );
  var prefIdentifier  = prefElements[ 1 ].getAttribute( "prefstring" );
  var prefDefaultFlag = true;
  var prefValue       = readMyPref( prefIdentifier, prefType, 0, prefDefaultFlag );

  for( var i = 1; i < prefElements.length; i++ ) {
    if ( prefElements[ i ].getAttribute( "prefvalue" ) == prefValue )
      prefElements[ i ].setAttribute( "checked", "true" );       
    else
      prefElements[ i ].removeAttribute( "checked" );
  }
}
////////////////////////////////////////////////////////////////////////////////
// this function saves a changed preference setting
////////////////////////////////////////////////////////////////////////////////
function changePref( menuElement, menuGroup, doRefresh )
{

  var prefType        = menuElement.getAttribute( "preftype" );
  var prefIdentifier  = menuElement.getAttribute( "prefstring" );
  var prefDefval      = menuElement.getAttribute( "prefdefval" );
  var prefAttribute   = null;
  var prefDefaultFlag = true;
  var prefArray       = new Array();
  var noAccess        = "noAccess";
  var allAccess       = "allAccess";
	
  if ( prefIdentifier != null ) {
    // retrieve current value
    var prefValue = readMyPref( prefIdentifier, prefType, prefDefval, prefDefaultFlag );

    if ( prefType == "bool" ) // flip flop the pref
      writePref( prefType, prefIdentifier, !prefValue, prefDefaultFlag );
    else {
      if ( menuGroup ) // special case, use this value
        prefValue = menuElement.getAttribute( "prefvalue" );
      else { // used for int and string, other might follow
        prefAttribute = menuElement.getAttribute( "prefattribute" );

        if ( prefAttribute == "checked" ) {
          // setup an array with values to use
          prefValues = eval( menuElement.getAttribute( "prefvalue" ) );
	        prefArray = prefValues;
          // pick the first or second value, based on the "checked" state
          if ( menuElement.getAttribute( prefAttribute ) )
            prefValue = prefArray[ 1 ];
          else prefValue = prefArray[ 0 ];
        }
        else
          prefValue = menuElement.getAttribute( prefAttribute );
      }
      // Ok, lets write this sucker
      writePref( prefType, prefIdentifier, prefValue, prefDefaultFlag );
    }
    if ( doRefresh ) // this pref setting needs a refresh
      getWebNavigation().reload( nsIWebNavigation.LOAD_FLAGS_IS_REFRESH );
  }
}
////////////////////////////////////////////////////////////////////////////////
// this function initializes the preferences for the menu
////////////////////////////////////////////////////////////////////////////////
function initQuickPrefs()
{
  var prefDefaultFlag = true;
  var prefElements    = document.getElementsByAttribute( "pref", "true" );

  for ( var i = 0; i < prefElements.length; i++ ) {
    var prefId         = prefElements[ i ].getAttribute( "id" );
    var prefType       = prefElements[ i ].getAttribute( "preftype" );
    var prefIdentifier = prefElements[ i ].getAttribute( "prefstring" );
    var prefDefval     = prefElements[ i ].getAttribute( "prefdefval" );
    var prefAttribute  = prefElements[ i ].getAttribute( "prefattribute" );
    var prefValue;

    prefValue = readMyPref( prefIdentifier, prefType, prefDefval, prefDefaultFlag );

    if ( prefAttribute == "checked" ) {
      if ( prefValue >= 1 )
        prefElements[ i ].setAttribute( prefAttribute, "true" );
      else prefElements[ i ].removeAttribute( prefAttribute );
    }
    else 
      prefElements[ i ].setAttribute( prefAttribute, prefValue );
  }
}
////////////////////////////////////////////////////////////////////////////////
// this function initializes the menu items for the Cookie/Image settings
//------------------------------------------------------------------------------
// this code is copied from cookieNavigatorOverlay.xul and changed accordantly
// and it is always available for the end-user. We don't check the pref :-)
////////////////////////////////////////////////////////////////////////////////
function aCheckForVisibility()
{
  var permissionmanager = Components.classes[ "@mozilla.org/permissionmanager;1" ]
                                    .getService(Components.interfaces.nsIPermissionManager );
  var blocked = permissionmanager.testForBlocking( window._content.location, COOKIEPERMISSION );
  enableElement("qpm_AllowCookies", blocked);
  enableElement("qpm_BlockCookies", !blocked);
  blocked = permissionmanager.testForBlocking(window._content.location, IMAGEPERMISSION);
  enableElement("qpm_AllowImages", blocked);
  enableElement("qpm_BlockImages", !blocked);

  var popupmanager = Components.classes["@mozilla.org/PopupWindowManager;1"]
                               .getService(Components.interfaces.nsIPopupWindowManager);
  var pref;
  pref = Components.classes[ '@mozilla.org/preferences-service;1' ];
  pref = pref.getService();
  pref = pref.QueryInterface( Components.interfaces.nsIPrefBranch );
  var useManager = !pref.getBoolPref("dom.disable_open_during_load") &&
                    pref.getIntPref("privacy.popups.policy") == Components.interfaces.nsIPopupWindowManager.ALLOW_POPUP &&
                    pref.getBoolPref("privacy.popups.usecustom");
  var suitable = false;
  var enableBlock = false;
  if (useManager) {
    suitable = popupmanager.testSuitability(getBrowser().currentURI);
    if (suitable)
      enableBlock = popupmanager.testPermission(getBrowser().currentURI) != Components.interfaces.nsIPopupWindowManager.DENY_POPUP;
  }
  enableElement("qpm_BlockPopups", useManager && suitable && enableBlock);
  enableElement("qpm_AllowPopups", useManager && suitable && !enableBlock);
  enableElement("qpm_ManagePopups", useManager);
}
///////////////////////////////////////////////////////////////////////////////
// This function enables/disables menu items for the Cookie/Image/Popup menu's 
//------------------------------------------------------------------------------
// This is a rewrite of aCookieImageAction() in cookieNavigatorOverlay.xul 
////////////////////////////////////////////////////////////////////////////////
function aCIPAction(element) {
  var permissionmanager = Components.classes[ "@mozilla.org/permissionmanager;1" ]
                                    .getService(Components.interfaces.nsIPermissionManager );
  var permission = (element.getAttribute("permission") == "true") ? true : false;
  var action = element.getAttribute("for")

  switch (action) {
    case "qpm_CookieAction":
      permissionmanager.add(window._content.location, permission, 0);
      break;
    case "qpm_ImageAction":
      permissionmanager.add(window._content.location, permission, 1);
      break;
    case "qpm_PopupAction":
      var popupmanager = Components.classes["@mozilla.org/PopupWindowManager;1"]
                                   .getService(Components.interfaces.nsIPopupWindowManager);
      popupmanager.add(getBrowser().currentURI, permission);
      break;
  }
  alert(element.getAttribute("msg"));
}
////////////////////////////////////////////////////////////////////////////////
// this clears the UAgent submenu and fills it with data from localstore.rdf
////////////////////////////////////////////////////////////////////////////////
// you need to add this to your copy of localstore.rdf in the profile directory
////////////////////////////////////////////////////////////////////////////////
//  <RDF:Seq about="nc:useragent-data">
//    <RDF:li>Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.0.0+) Gecko/20020518</RDF:li>
//    <RDF:li>Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.0rc2) Gecko/20020510</RDF:li>
//    <RDF:li>Mozilla/5.0 (Windows; U; Win98; en-US; rv:0.9.9) Gecko/20020311</RDF:li>
//    <RDF:li>Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.0rc2) Gecko/20020513 Netscape/7.0b1</RDF:li>
//    <RDF:li>Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.0rc2) Gecko/20020512 Netscape/7.0b1</RDF:li>
//    <RDF:li>Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.4) Gecko/20011022 Netscape6/6.2</RDF:li>
//    <RDF:li>Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)</RDF:li>
//    <RDF:li>Mozilla/4.0 (compatible; MSIE 5.5; Windows 98)</RDF:li>
//    <RDF:li>Mozilla/4.0 (compatible; MSIE 5.0; Mac_PowerPC)</RDF:li>
//    <RDF:li>Mozilla/4.78 [en] (Win98; U)</RDF:li>
//    <RDF:li>Mozilla/4.78 (Macintosh; U; PPC)</RDF:li>
//    <RDF:li>Mozilla/4.78 [en] (X11; U; Linux 2.2.19 i686)</RDF:li>
//  </RDF:Seq>
////////////////////////////////////////////////////////////////////////////////
function getUserAgentFromLocalstore(aParent)
{
  var uaOverride = null;
  try { 
    uaOverride = pref.getCharPref("general.useragent.override"); 
  } catch(ex) {}
  document.getElementById("quickpref-defaultUserAgent").setAttribute("disabled", uaOverride ? "false" : "true");
  deleteHistoryItems(aParent);
  var index = 0;
  if (!gdLocalStore)
    gdLocalStore = RDF.GetDataSource("rdf:local-store");
  /* XXX: Added for Phoenix */
  var RDFC = Components.classes["@mozilla.org/rdf/container-utils;1"]
                       .getService(Components.interfaces.nsIRDFContainerUtils);
  var userAgents = RDFC.MakeSeq(gdLocalStore, RDF.GetResource("nc:useragent-data")).GetElements();

  if (!userAgents.hasMoreElements())
    return(index);

  while (userAgents.hasMoreElements()) {
    var userAgent = userAgents.getNext();
    if (userAgent) {
      try {
        userAgent = userAgent.QueryInterface(Components.interfaces.nsIRDFLiteral);
        index++;
      } catch(ex) {
        continue;
      }
     createMenuItem(aParent, index, userAgent.Value);
    }
  }
  return(index);
}
////////////////////////////////////////////////////////////////////////////////
// this function stores the selected useragent in 'general.useragent.override'
////////////////////////////////////////////////////////////////////////////////
function setUserAgentOverrideTo(index, aIdentifier)
{
  if (!index) {
    writePref("char", "general.useragent.override", null);
    pref.clearUserPref("general.useragent.override");
  }
  else 
    writePref("char", "general.useragent.override", aIdentifier);
  loadURI("about:");
}
////////////////////////////////////////////////////////////////////////////////
// this function initializes the referrer spoofer menu items
////////////////////////////////////////////////////////////////////////////////
function initRefSpoofingMenu(aElement)
{
  var refSetting = -1;
  var spoofURL = "http://ref.spoof.org/refspoof.cgi"; // default fake referrer
  var aWindow = gBrowser.contentWindow;
  var aDocument = aWindow.document;
  var qpmBundle = srGetStrBundle("chrome://multiviews/locale/quickPrefsMenu.properties");
  try { 
    refSetting = pref.getIntPref("network.http.sendRefererHeader");
  } catch (ex) {
  }
  for (var i = 0; i < aElement.childNodes.length; i++) {
    aElement.childNodes[i].setAttribute("disabled", (refSetting == 0) ? "true" : "false");
    aElement.childNodes[i].setAttribute("checked", "false");
  }
  try {
    /* spoofURL = pref.getComplexValue("multizilla.refspoof.spoofurl",
                                    Components.interfaces.nsIPrefLocalizedString).data; */
    spoofURL = readMyPref("multizilla.refspoof.spoofurl", "string", spoofURL );
  } catch (ex) {
  }
  aElement.childNodes[6].setAttribute("label", qpmBundle.GetStringFromName("fake.label") + " " + spoofURL );
  aElement.childNodes[7].setAttribute("label", aDocument.referrer ?  
                                               qpmBundle.GetStringFromName("currentRef.label") + " " + aDocument.referrer : 
                                               qpmBundle.GetStringFromName("noRef.label") );

  if (refSetting)
    initGroupPref(aElement);
}
////////////////////////////////////////////////////////////////////////////////
// this function 
////////////////////////////////////////////////////////////////////////////////
function manageFakeReferrer()
{
  alert("Sorry, not yet implemented");
  return;

  try {
    spoofURL = pref.getComplexValue("multizilla.refspoof.spoofurl",
                                    Components.interfaces.nsIPrefLocalizedString).data;
    referrer = ioService.newURI(spoofURL, null, null);
  } catch (ex) {
  }
}

//////////////////////////////////////////////////////////////////////////
// this will bring up a requester asking if the user actually wants to
// perform an action
//////////////////////////////////////////////////////////////////////////
function confirmAction(titlePtr, textPtr)
{
  var qpmBundle= srGetStrBundle("chrome://multiviews/locale/quickPrefsMenu.properties");
  var aDialogTitle= qpmBundle.GetStringFromName(titlePtr);
  var aDialogText= qpmBundle.GetStringFromName(textPtr);
  var promptService= Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService();
  promptService= promptService.QueryInterface(Components.interfaces.nsIPromptService);
  return promptService.confirm(window, aDialogTitle, aDialogText);
} // end function confirmSomething()

//////////////////////////////////////////////////////////////////////////
// wrapper function to clear both disk and memory cache in one go
//////////////////////////////////////////////////////////////////////////
function clearDiskMemCache()
{
  clearCache('STORE_ON_DISK');
  clearCache('STORE_IN_MEMORY');
}
////////////////////////////////////////////////////////////////////////////////
// this feature is implemented to clear both memory and disk-cache as well as 
// (url) history info, on special request by some anonymous dirty people, haha
// sorry, but I had to add this type of comment, just for the fun part of it :-)
////////////////////////////////////////////////////////////////////////////////
function clearAllBackTraces()
{
  clearGlobalHistory();
  clearUrlBarHistory();
  clearCache("STORE_ON_DISK");
  clearCache("STORE_IN_MEMORY");
}
////////////////////////////////////////////////////////////////////////////////
// this function clears the global history
////////////////////////////////////////////////////////////////////////////////
function clearGlobalHistory()
{
  var globalHistory = Components.classes["@mozilla.org/browser/global-history;1"]
                                .getService(Components.interfaces.nsIBrowserHistory);
  globalHistory.removeAllPages();
}
////////////////////////////////////////////////////////////////////////////////
// this function clears the global history
////////////////////////////////////////////////////////////////////////////////
function clearUrlBarHistory()
{
  var urlBarHistory = Components.classes["@mozilla.org/browser/urlbarhistory;1"]
                                .getService(Components.interfaces.nsIUrlbarHistory);
  urlBarHistory.clearHistory();
}
////////////////////////////////////////////////////////////////////////////////
// this function clears the global history
////////////////////////////////////////////////////////////////////////////////
function clearCache(aCache)
{
  var classID       = Components.classes["@mozilla.org/network/cache-service;1"];
  var cacheService  = classID.getService(Components.interfaces.nsICacheService);

  if (aCache == "STORE_ON_DISK")
    cacheService.evictEntries(Components.interfaces.nsICache.STORE_ON_DISK);
  else if (aCache == "STORE_IN_MEMORY")
    cacheService.evictEntries(Components.interfaces.nsICache.STORE_IN_MEMORY);
}